@@ -20,6 +20,10 @@
 
 #include <gio/gio.h>
 
-void	pt_i18n_init	(void);
+G_BEGIN_DECLS
+
+void	_pt_i18n_init	(void);
+
+G_END_DECLS
 
 #endif
