@@ -57,6 +57,7 @@ struct _PtPlayerPrivate
 	PtPositionManager *pos_mgr;
 	GHashTable *plugins;
 
+	PtStateType state;
 	gint64	    dur;
 	gdouble	    speed;
 	gdouble     volume;
@@ -84,6 +85,8 @@ struct _PtPlayerPrivate
 enum
 {
 	PROP_0,
+	PROP_STATE,
+	PROP_CURRENT_URI,
 	PROP_SPEED,
 	PROP_MUTE,
 	PROP_VOLUME,
@@ -161,7 +164,7 @@ pt_player_get_file (PtPlayer *player)
 	gchar *uri = NULL;
 	GFile *result = NULL;
 
-	g_object_get (G_OBJECT (player->priv->play), "current_uri", &uri, NULL);
+	g_object_get (G_OBJECT (player->priv->play), "current-uri", &uri, NULL);
 
 	if (uri) {
 		result = g_file_new_for_uri (uri);
@@ -272,6 +275,26 @@ bus_call (GstBus     *bus,
 	case GST_MESSAGE_STATE_CHANGED: {
 		if (GST_MESSAGE_SRC(msg) != GST_OBJECT (player->priv->play))
 			break;
+		GstState old_state, new_state;
+		gst_message_parse_state_changed (msg, &old_state, &new_state, NULL);
+		if (old_state != new_state) {
+			if (new_state == GST_STATE_PLAYING) {
+				player->priv->state = PT_STATE_PLAYING;
+				g_object_notify_by_pspec (G_OBJECT (player),
+					                  obj_properties[PROP_STATE]);
+			}
+			if (new_state == GST_STATE_PAUSED) {
+				player->priv->state = PT_STATE_PAUSED;
+				g_object_notify_by_pspec (G_OBJECT (player),
+					                  obj_properties[PROP_STATE]);
+			}
+			if (new_state <= GST_STATE_READY &&
+			    old_state >= GST_STATE_PAUSED) {
+				player->priv->state = PT_STATE_STOPPED;
+				g_object_notify_by_pspec (G_OBJECT (player),
+					                  obj_properties[PROP_STATE]);
+			}
+		}
 
 		gdouble volume;
 		volume = gst_stream_volume_get_volume (GST_STREAM_VOLUME (player->priv->audio_bin),
@@ -772,22 +795,40 @@ pt_player_jump_to_position (PtPlayer *player,
 		return;
 	}
 
-	/* TODO on opening a new file and jumping to the initial position,
-	 * sometimes there is no duration yet and the jump is not done. */
-
-	if (pos > player->priv->segend || pos < player->priv->segstart) {
+	if (pos < player->priv->segstart) {
 		g_log_structured (G_LOG_DOMAIN, G_LOG_LEVEL_DEBUG,
-				  "MESSAGE", "Jump to position failed: start = %" G_GINT64_FORMAT, player->priv->segstart);
+		                  "MESSAGE", "Setting position failed: target %" G_GINT64_FORMAT
+				  " before segstart %" G_GINT64_FORMAT, pos, player->priv->segstart);
+		return;
+	}
+	if (player->priv->segend != -1 && pos > player->priv->segend) {
 		g_log_structured (G_LOG_DOMAIN, G_LOG_LEVEL_DEBUG,
-				  "MESSAGE", "Jump to position failed: pos   = %" G_GINT64_FORMAT, pos);
-		g_log_structured (G_LOG_DOMAIN, G_LOG_LEVEL_DEBUG,
-				  "MESSAGE", "Jump to position failed: end   = %" G_GINT64_FORMAT, player->priv->segend);
+		                  "MESSAGE", "Setting position failed: target %" G_GINT64_FORMAT
+				  " after segend %" G_GINT64_FORMAT, pos, player->priv->segend);
 		return;
 	}
 
 	pt_player_seek (player, pos);
 }
 
+/**
+ * pt_player_get_speed:
+ * @player: a #PtPlayer
+ *
+ * Returns current playback speed.
+ *
+ * Return value: playback speed
+ *
+ * Since: 3.1
+ */
+gdouble
+pt_player_get_speed (PtPlayer *player)
+{
+	g_return_val_if_fail (PT_IS_PLAYER (player), 1);
+
+	return player->priv->speed;
+}
+
 /**
  * pt_player_set_speed:
  * @player: a #PtPlayer
@@ -1075,7 +1116,7 @@ pt_player_get_uri (PtPlayer *player)
 	g_return_val_if_fail (PT_IS_PLAYER (player), NULL);
 
 	gchar *uri = NULL;
-	g_object_get (G_OBJECT (player->priv->play), "current_uri", &uri, NULL);
+	g_object_get (G_OBJECT (player->priv->play), "current-uri", &uri, NULL);
 	return uri;
 }
 
@@ -1663,6 +1704,24 @@ mute_changed (GObject    *object,
 	g_source_set_name_by_id (id, "[parlatype] notify_mute_idle_cb");
 }
 
+static gboolean
+notify_uri_idle_cb (PtPlayer *player)
+{
+	g_object_notify_by_pspec (G_OBJECT (player),
+	                          obj_properties[PROP_CURRENT_URI]);
+	return FALSE;
+}
+
+static void
+uri_changed (GObject    *object,
+             GParamSpec *pspec,
+             PtPlayer   *player)
+{
+	guint id;
+	id = g_idle_add ((GSourceFunc) notify_uri_idle_cb, player);
+	g_source_set_name_by_id (id, "[parlatype] notify_uri_idle_cb");
+}
+
 /**
  * pt_player_configure_asr:
  * @player: a #PtPlayer
@@ -1888,8 +1947,17 @@ pt_player_get_property (GObject    *object,
                         GParamSpec *pspec)
 {
 	PtPlayer *player = PT_PLAYER (object);
+	char *uri;
 
 	switch (property_id) {
+	case PROP_STATE:
+		g_value_set_int (value, player->priv->state);
+		break;
+	case PROP_CURRENT_URI:
+		g_object_get (G_OBJECT (player->priv->play), "current-uri", &uri, NULL);
+		g_value_set_string (value, uri);
+		g_free (uri);
+		break;
 	case PROP_SPEED:
 		g_value_set_double (value, player->priv->speed);
 		break;
@@ -1986,6 +2054,11 @@ pt_player_init (PtPlayer *player)
 			"notify::volume", G_CALLBACK (vol_changed), player);
 	g_signal_connect (G_OBJECT (priv->play),
 			"notify::mute", G_CALLBACK (mute_changed), player);
+
+	/* Forward current-uri changes */
+	g_signal_connect (G_OBJECT (priv->play),
+			"notify::current-uri", G_CALLBACK (uri_changed), player);
+
 }
 
 static void
@@ -2305,6 +2378,34 @@ pt_player_class_init (PtPlayerClass *klass)
 			FALSE,
 			G_PARAM_READWRITE);
 
+	/**
+	* PtPlayer:state:
+	*
+	* The current state of PtPlayer.
+	*/
+	obj_properties[PROP_STATE] =
+	g_param_spec_int (
+			"state",
+			"State",
+			"PtPlayer’s current state",
+			0,	/* minimum = PT_MODE_EMPTY */
+			4,	/* maximum = PT_MODE_INVALID */
+			0,
+			G_PARAM_READABLE);
+
+	/**
+	* PtPlayer:current-uri:
+	*
+	* URI of the currently loaded stream.
+	*/
+	obj_properties[PROP_CURRENT_URI] =
+	g_param_spec_string (
+			"current-uri",
+			"Current URI",
+			"URI of the currently loaded stream",
+			NULL,
+			G_PARAM_READABLE);
+
 	g_object_class_install_properties (
 			G_OBJECT_CLASS (klass),
 			N_PROPERTIES,
