@@ -54,6 +54,23 @@ struct _PtPlayerClass
 	GObjectClass parent_class;
 };
 
+/**
+ * PtStateType:
+ * @PT_STATE_STOPPED: the player is stopped
+ * @PT_STATE_PAUSED: the player is paused
+ * @PT_STATE_PLAYING: the player is currently playing a stream
+ *
+ * Enum values indicating PtPlayer’s current state.
+ */
+typedef enum {
+	PT_STATE_STOPPED,
+	PT_STATE_PAUSED,
+	PT_STATE_PLAYING,
+	/*< private >*/
+	PT_STATE_INVALID
+} PtStateType;
+
+
 /**
  * PtModeType:
  * @PT_MODE_PLAYBACK: normal audible playback
@@ -107,6 +124,7 @@ gint		pt_player_get_back		(PtPlayer *player);
 gint		pt_player_get_forward		(PtPlayer *player);
 void		pt_player_jump_to_position	(PtPlayer *player,
 						 gint      milliseconds);
+gdouble		pt_player_get_speed		(PtPlayer *player);
 void		pt_player_set_speed		(PtPlayer *player,
 						 gdouble   speed);
 gdouble		pt_player_get_volume		(PtPlayer *player);
