@@ -1,6 +1,6 @@
 /* Copyright (C) 2006-2008 Buzztrax team <buzztrax-devel@buzztrax.org>
  * Copyright (C) 2002 2003 2004 2005 2007 2008 2009 2010, Magnus Hjorth
- * Copyright (C) 2016 Gabor Karsay <gabor.karsay@gmx.at>
+ * Copyright (C) 2016, 2020 Gabor Karsay <gabor.karsay@gmx.at>
  *
  * Original source name waveform-viewer.c, taken from Buzztrax and heavily
  * modified. Original source licenced under LGPL 2 or later.
@@ -23,6 +23,17 @@
  * with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
+/**
+ * SECTION: pt-waveviewer
+ * @short_description: A GtkWidget to display a waveform.
+ * @stability: Stable
+ * @include: parlatype/pt-waveviewer.h
+ *
+ * Displays a waveform and lets the user interact with it: jumping to a position,
+ * make selections and so on.
+ */
+
+
 #include "config.h"
 #include <math.h>	/* fabs */
 #include <string.h>
@@ -32,6 +43,7 @@
 #include "pt-waveloader.h"
 #include "pt-waveviewer-ruler.h"
 #include "pt-waveviewer-waveform.h"
+#include "pt-waveviewer-scrollbox.h"
 #include "pt-waveviewer-selection.h"
 #include "pt-waveviewer-cursor.h"
 #include "pt-waveviewer-focus.h"
@@ -56,7 +68,6 @@ struct _PtWaveviewerPrivate {
 	gboolean  fixed_cursor;
 	gboolean  show_ruler;
 	gboolean  has_selection;
-	gboolean  zoom;
 	gint      pps;
 
 	gint64    zoom_time;
@@ -70,10 +81,11 @@ struct _PtWaveviewerPrivate {
 	GdkCursor *arrows;
 
 	GtkAdjustment   *adj;
-	gboolean         rtl;
 	gboolean         focus_on_cursor;
 
 	/* Subwidgets */
+	GtkWidget  *scrollbox;
+	GtkWidget  *overlay;
 	GtkWidget  *waveform;
 	GtkWidget  *revealer;
 	GtkWidget  *ruler;
@@ -121,38 +133,6 @@ static guint signals[LAST_SIGNAL] = { 0 };
 G_DEFINE_TYPE_WITH_PRIVATE (PtWaveviewer, pt_waveviewer, GTK_TYPE_SCROLLED_WINDOW);
 
 
-/**
- * SECTION: pt-waveviewer
- * @short_description: A GtkWidget to display a waveform.
- * @stability: Stable
- * @include: parlatype/pt-waveviewer.h
- *
- * Displays a waveform and lets the user interact with it: jumping to a position,
- * make selections and so on.
- */
-
-
-static gint64
-flip_pixel (PtWaveviewer *self,
-            gint64        pixel)
-{
-	/* In ltr layouts each pixel on the x-axis corresponds to a sample in the array.
-	   In rtl layouts this is flipped, e.g. the first pixel corresponds to
-	   the last sample in the array. */
-
-	gint64 samples;
-	gint   widget_width;
-
-	samples = self->priv->peaks_size / 2;
-	widget_width = gtk_widget_get_allocated_width (self->priv->waveform);
-
-	/* Case: waveform is shorter than drawing area */
-	if (samples < widget_width)
-		return (widget_width - pixel);
-	else
-		return (samples - pixel);
-}
-
 static gint64
 time_to_pixel (PtWaveviewer *self,
                gint64        ms)
@@ -163,9 +143,6 @@ time_to_pixel (PtWaveviewer *self,
 	result = ms * self->priv->px_per_sec;
 	result = result / 1000;
 
-	if (self->priv->rtl)
-		result = flip_pixel (self, result);
-
 	return result;
 }
 
@@ -176,9 +153,6 @@ pixel_to_time (PtWaveviewer *self,
 	/* Convert a position in the drawing area to time in milliseconds */
 	gint64 result;
 
-	if (self->priv->rtl)
-		pixel = flip_pixel (self, pixel);
-
 	result = pixel * 1000;
 	result = result / self->priv->px_per_sec;
 
@@ -207,16 +181,25 @@ scroll_to_cursor (PtWaveviewer *self)
 		gtk_adjustment_set_value (self->priv->adj, cursor_pos - offset);
 	} else {
 		if (cursor_pos < first_visible || cursor_pos > last_visible) {
-			if (self->priv->rtl)
-				/* cursor visible far right */
-				gtk_adjustment_set_value (self->priv->adj, cursor_pos - page_width);
-			else
-				/* cursor visible far left */
-				gtk_adjustment_set_value (self->priv->adj, cursor_pos);
+			/* cursor visible far left */
+			gtk_adjustment_set_value (self->priv->adj, cursor_pos);
 		}
 	}
 }
 
+static void
+render_cursor (PtWaveviewer *self)
+{
+	PtWaveviewerCursor *cursor;
+	gint offset, pixel;
+
+	cursor = PT_WAVEVIEWER_CURSOR (self->priv->cursor);
+	offset = gtk_adjustment_get_value (self->priv->adj);
+	pixel = time_to_pixel (self, self->priv->playback_cursor);
+
+	pt_waveviewer_cursor_render (cursor, pixel - offset);
+}
+
 static gint64
 calculate_duration (PtWaveviewer *self)
 {
@@ -243,8 +226,6 @@ add_subtract_time (PtWaveviewer *self,
 	gint   one_pixel;
 
 	one_pixel = 1000 / self->priv->px_per_sec;
-	if (self->priv->rtl)
-		one_pixel = one_pixel * -1;
 	result = self->priv->playback_cursor + pixel * one_pixel;
 
 	/* Return result in range */
@@ -407,16 +388,16 @@ pt_waveviewer_key_press_event (GtkWidget   *widget,
 				scroll = GTK_SCROLL_STEP_FORWARD;
 				break;
 			case GDK_KEY_Page_Up:
-				scroll = self->priv->rtl ? GTK_SCROLL_PAGE_FORWARD : GTK_SCROLL_PAGE_BACKWARD;
+				scroll = GTK_SCROLL_PAGE_BACKWARD;
 				break;
 			case GDK_KEY_Page_Down:
-				scroll = self->priv->rtl ? GTK_SCROLL_PAGE_BACKWARD : GTK_SCROLL_PAGE_FORWARD;
+				scroll = GTK_SCROLL_PAGE_FORWARD;
 				break;
 			case GDK_KEY_Home:
-				scroll = self->priv->rtl ? GTK_SCROLL_END : GTK_SCROLL_START;
+				scroll = GTK_SCROLL_START;
 				break;
 			case GDK_KEY_End:
-				scroll = self->priv->rtl ? GTK_SCROLL_START : GTK_SCROLL_END;
+				scroll = GTK_SCROLL_END;
 				break;
 			}
 			if (scroll != GTK_SCROLL_NONE) {
@@ -471,7 +452,7 @@ pt_waveviewer_button_press_event (GtkGestureMultiPress *gesture,
 
 	button = gtk_gesture_single_get_current_button (GTK_GESTURE_SINGLE (gesture));
 	clicked = (gint) x;
-	pos = pixel_to_time (self, clicked);
+	pos = pixel_to_time (self, clicked + gtk_adjustment_get_value (self->priv->adj));
 
 	/* Single left click, no other keys pressed: new selection or changing selection */
 	if (n_press == 1
@@ -481,10 +462,10 @@ pt_waveviewer_button_press_event (GtkGestureMultiPress *gesture,
 		self->priv->dragstart = self->priv->dragend = pos;
 
 		/* if over selection border: snap to selection border, changing selection */
-		if (pointer_in_range (self, x, self->priv->sel_start)) {
+		if (pointer_in_range (self, x + gtk_adjustment_get_value (self->priv->adj), self->priv->sel_start)) {
 			self->priv->dragstart = self->priv->sel_end;
 			self->priv->dragend = self->priv->sel_start;
-		} else if (pointer_in_range (self, x, self->priv->sel_end)) {
+		} else if (pointer_in_range (self, x + gtk_adjustment_get_value (self->priv->adj), self->priv->sel_end)) {
 			self->priv->dragstart = self->priv->sel_start;
 			self->priv->dragend = self->priv->sel_end;
 		}
@@ -587,7 +568,7 @@ pt_waveviewer_motion_notify_event (GtkWidget      *widget,
 		return FALSE;
 
 	clicked = (gint) x;
-	pos = pixel_to_time (self, clicked);
+	pos = pixel_to_time (self, clicked + gtk_adjustment_get_value (self->priv->adj));
 
 	/* Right mouse button sets cursor */
 	if (state & GDK_BUTTON3_MASK) {
@@ -604,8 +585,8 @@ pt_waveviewer_motion_notify_event (GtkWidget      *widget,
 
 	/* No button or any other button: change pointer cursor over selection border */
 	if (self->priv->sel_start != self->priv->sel_end) {
-		if (pointer_in_range (self, x, self->priv->sel_start)
-		    || pointer_in_range (self, x, self->priv->sel_end)) {
+		if (pointer_in_range (self, x + gtk_adjustment_get_value (self->priv->adj), self->priv->sel_start)
+		    || pointer_in_range (self, x + gtk_adjustment_get_value (self->priv->adj), self->priv->sel_end)) {
 			set_cursor (GTK_WIDGET (self), self->priv->arrows);
 		} else {
 			set_cursor (GTK_WIDGET (self), NULL);
@@ -632,76 +613,6 @@ pt_waveviewer_button_release_event (GtkGestureMultiPress *gesture,
 	return FALSE;
 }
 
-static void
-update_cached_style_values (PtWaveviewer *self)
-{
-	/* Only direction is cached */
-
-	GtkStyleContext *context;
-	GtkStateFlags    state;
-	GdkWindow       *window = NULL;
-
-	window = gtk_widget_get_parent_window (GTK_WIDGET (self));
-	if (!window)
-		return;
-
-	context = gtk_widget_get_style_context (GTK_WIDGET (self));
-	state = gtk_style_context_get_state (context);
-
-	if (state & GTK_STATE_FLAG_DIR_RTL)
-		self->priv->rtl = TRUE;
-	else
-		self->priv->rtl = FALSE;
-}
-
-static void
-pt_waveviewer_state_flags_changed (GtkWidget     *widget,
-                                   GtkStateFlags  flags)
-{
-	update_cached_style_values (PT_WAVEVIEWER (widget));
-	GTK_WIDGET_CLASS (pt_waveviewer_parent_class)->state_flags_changed (widget, flags);
-}
-
-static void
-pt_waveviewer_style_updated (GtkWidget *widget)
-{
-	PtWaveviewer *self = PT_WAVEVIEWER (widget);
-	GTK_WIDGET_CLASS (pt_waveviewer_parent_class)->style_updated (widget);
-	update_cached_style_values (self);
-}
-
-static void
-pt_waveviewer_direction_changed (GtkWidget        *widget,
-                                 GtkTextDirection  direction)
-{
-	PtWaveviewer *self = PT_WAVEVIEWER (widget);
-	gint widget_width, first_visible, page_width;
-
-	/* This is called after state-flags-changed signal.
-	   The waveform handles repainting itself, here we have to flip
-	   the GtkAdjustment and recalculate the cursor’s position */
-	widget_width = gtk_widget_get_allocated_width (self->priv->waveform);
-	first_visible = (gint) gtk_adjustment_get_value (self->priv->adj);
-	page_width = (gint) gtk_adjustment_get_page_size (self->priv->adj);
-	gtk_adjustment_set_value (
-			self->priv->adj,
-			widget_width - first_visible - page_width);
-
-	if (self->priv->peaks)
-		pt_waveviewer_cursor_render (
-				PT_WAVEVIEWER_CURSOR (self->priv->cursor),
-				time_to_pixel (self, self->priv->playback_cursor));
-
-	GTK_WIDGET_CLASS (pt_waveviewer_parent_class)->direction_changed (widget, direction);
-}
-
-static void
-pt_waveviewer_realize (GtkWidget *widget)
-{
-	GTK_WIDGET_CLASS (pt_waveviewer_parent_class)->realize (widget);
-	update_cached_style_values (PT_WAVEVIEWER (widget));
-}
-
 static void
 stop_following_cursor (PtWaveviewer *self)
 {
@@ -882,32 +793,6 @@ cursor_is_visible (PtWaveviewer *self)
 	return (left <= cursor_pos && cursor_pos <= right);
 }
 
-static void
-size_allocate_cb (GtkWidget    *widget,
-                  GdkRectangle *allocation,
-                  gpointer      user_data)
-{
-	PtWaveviewer *self = PT_WAVEVIEWER (user_data);
-
-	if (self->priv->zoom) {
-		gtk_adjustment_set_value (
-				self->priv->adj,
-				time_to_pixel (self, self->priv->zoom_time) - self->priv->zoom_pos);
-		self->priv->zoom = FALSE;
-	}
-
-	if (self->priv->peaks->len > 0) {
-		pt_waveviewer_cursor_render (
-				PT_WAVEVIEWER_CURSOR (self->priv->cursor),
-				time_to_pixel (self, self->priv->playback_cursor));
-	}
-
-	pt_waveviewer_selection_set (
-			PT_WAVEVIEWER_SELECTION (self->priv->selection),
-			time_to_pixel (self, self->priv->sel_start),
-			time_to_pixel (self, self->priv->sel_end));
-}
-
 static void
 get_anchor_point (PtWaveviewer *self)
 {
@@ -927,115 +812,39 @@ get_anchor_point (PtWaveviewer *self)
 	}
 }
 
-
-#if 0
 static void
-make_widget_ready (PtWaveviewer *self,
-                   gboolean      ready)
+reset_selection (PtWaveviewer *self)
 {
-	gint widget_width, left;
+	PtWaveviewerPrivate *priv = self->priv;
 
-	if (!ready && self->priv->zoom) {
-		/* If this is a zoom, we still have the old waveform.
-		   Get an anchor point = zoom_pos, either cursor position or
-		   middle of view. The anchor point is relative to the view.
-		   Get time of that anchor point = zoom_time. */
-
-		left = (gint) gtk_adjustment_get_value (self->priv->adj);
-		if (cursor_is_visible (self)) {
-			self->priv->zoom_time = self->priv->playback_cursor;
-			self->priv->zoom_pos = time_to_pixel (self, self->priv->playback_cursor) - left;
-		} else {
-			self->priv->zoom_pos = (gint) gtk_adjustment_get_page_size (self->priv->adj) / 2;
-			self->priv->zoom_time = pixel_to_time (self, self->priv->zoom_pos + left);
-		}
-	}
-
-	if (!ready && !self->priv->zoom) {
-		/* Reset previous selections */
-		self->priv->sel_start = self->priv->sel_end = 0;
-		self->priv->has_selection = FALSE;
-		g_object_notify_by_pspec (
-				G_OBJECT (self),
-				obj_properties[PROP_SELECTION_START]);
-		g_object_notify_by_pspec (
-				G_OBJECT (self),
-				obj_properties[PROP_SELECTION_END]);
-		g_object_notify_by_pspec (
-				G_OBJECT (self),
-				obj_properties[PROP_HAS_SELECTION]);
-	}
-
-	if (!ready) {
-		if (self->priv->peaks) {
-			g_array_unref (self->priv->peaks);
-			self->priv->peaks = NULL;
-		}
-		self->priv->peaks_size = 0;
-		self->priv->px_per_sec = 0;
-		self->priv->duration = 0;
-		pt_waveviewer_selection_set (
-				PT_WAVEVIEWER_SELECTION (self->priv->selection),
-				0, 0);
-		pt_waveviewer_cursor_render (
-				PT_WAVEVIEWER_CURSOR (self->priv->cursor),
-				-1);
-	} else {
-		self->priv->duration = calculate_duration (self);
-		widget_width = self->priv->peaks_size / 2;
-		gtk_adjustment_set_upper (self->priv->adj, widget_width);
-		gtk_widget_set_size_request (
-				self->priv->waveform,
-				widget_width,
-				WAVE_MIN_HEIGHT);
-	}
-
-	pt_waveviewer_ruler_set_ruler (
-			PT_WAVEVIEWER_RULER (self->priv->ruler),
-			self->priv->peaks_size / 2,
-			self->priv->px_per_sec,
-			self->priv->duration);
-
-	/* Here we would render the cursor and change the GtkAdjustment so that
-	   we show the intended part of the waveform after zooming.
-	   However, there were some issues because the waveform didn’t
-	   allocate the new size in time. This is now done in the callback
-	   above, size_allocate_cb(). */
-}
-
-/**
- * pt_waveviewer_set_wave:
- * @self: the widget
- * @data: (nullable): a #PtWavedata
- *
- * Set wave data to show in the widget. The data is copied internally and may
- * be freed immediately after calling this function. If @data is NULL, a blank
- * widget is drawn.
- */
-void
-pt_waveviewer_set_wave (PtWaveviewer *self,
-                        GArray       *data)
-{
-	g_return_if_fail (PT_IS_WAVEVIEWER (self));
-
-	make_widget_ready (self, FALSE);
-	if (data == NULL || data->len == 0)
+	if (priv->has_selection == FALSE)
 		return;
 
-	self->priv->peaks = g_array_ref (data);
-	self->priv->peaks_size = data->len;
-	self->priv->px_per_sec = self->priv->pps;
+	g_object_freeze_notify (G_OBJECT (self));
 
-	make_widget_ready (self, TRUE);
+	priv->has_selection = FALSE;
+	g_object_notify_by_pspec (G_OBJECT (self),
+	                          obj_properties[PROP_HAS_SELECTION]);
+
+	if (priv->sel_start > 0) {
+		priv->sel_start = 0;
+		g_object_notify_by_pspec (G_OBJECT (self),
+		                          obj_properties[PROP_SELECTION_START]);
+	}
+
+	priv->sel_end = 0;
+	g_object_notify_by_pspec (G_OBJECT (self),
+	                          obj_properties[PROP_SELECTION_END]);
+
+	g_object_thaw_notify (G_OBJECT (self));
 }
-#endif
 
 static void
 array_size_changed_cb (PtWaveloader *loader,
 		       gpointer      user_data)
 {
 	PtWaveviewer *self = PT_WAVEVIEWER (user_data);
-	gint widget_width;
+	gint width;
 
 	if (self->priv->peaks == NULL || self->priv->peaks->len == 0) {
 		self->priv->peaks_size = 0;
@@ -1051,14 +860,11 @@ array_size_changed_cb (PtWaveloader *loader,
 		self->priv->peaks_size = self->priv->peaks->len;
 		self->priv->px_per_sec = self->priv->pps;
 		self->priv->duration = calculate_duration (self);
-		widget_width = self->priv->peaks_size / 2;
-		gtk_adjustment_set_upper (self->priv->adj, widget_width);
-		gtk_widget_set_size_request (
-				self->priv->waveform,
-				widget_width,
-				WAVE_MIN_HEIGHT);
 	}
 
+	width = self->priv->peaks_size / 2;
+	pt_waveviewer_scrollbox_set (PT_WAVEVIEWER_SCROLLBOX (self->priv->scrollbox), width);
+
 	pt_waveviewer_ruler_set_ruler (
 			PT_WAVEVIEWER_RULER (self->priv->ruler),
 			self->priv->peaks_size / 2,
@@ -1092,9 +898,9 @@ load_cb (PtWaveloader *loader,
 	if (pt_waveloader_load_finish (loader, res, &error)) {
 		array_size_changed_cb (NULL, self);
 		gtk_widget_queue_draw (self->priv->waveform);
+		render_cursor (self);
 		g_task_return_boolean (task, TRUE);
 	} else {
-		g_array_set_size (self->priv->peaks, 0);
 		array_size_changed_cb (NULL, self);
 		g_task_return_error (task, error);
 	}
@@ -1200,6 +1006,8 @@ pt_waveviewer_load_wave_async (PtWaveviewer        *self,
 	g_free (location);
 #endif
 
+	reset_selection (self);
+
 	g_object_set (self->priv->loader, "uri", uri, NULL);
 	self->priv->px_per_sec = self->priv->pps;
 	if (self->priv->tick_handler == 0) {
@@ -1221,7 +1029,6 @@ pt_waveviewer_finalize (GObject *object)
 {
 	PtWaveviewer *self = PT_WAVEVIEWER (object);
 
-	g_array_unref (self->priv->peaks);
 	g_clear_object (&self->priv->arrows);
 	g_clear_object (&self->priv->loader);
 	if (self->priv->tick_handler != 0) {
@@ -1305,8 +1112,7 @@ pt_waveviewer_set_property (GObject      *object,
 		if (self->priv->follow_cursor)
 			scroll_to_cursor (self);
 
-		pt_waveviewer_cursor_render (PT_WAVEVIEWER_CURSOR (self->priv->cursor),
-					     time_to_pixel (self, self->priv->playback_cursor));
+		render_cursor (self);
 		break;
 	case PROP_FOLLOW_CURSOR:
 		self->priv->follow_cursor = g_value_get_boolean (value);
@@ -1327,12 +1133,22 @@ pt_waveviewer_set_property (GObject      *object,
 		if (self->priv->peaks->len == 0)
 			break;
 		get_anchor_point (self);
-		self->priv->zoom = TRUE;
 		if (!pt_waveloader_resize (self->priv->loader,
 				           self->priv->pps,
 				           &error)) {
 			g_print ("%s\n", error->message);
 			g_clear_error (&error);
+			break;
+		}
+
+		array_size_changed_cb (NULL, self);
+		gtk_adjustment_set_value (self->priv->adj, time_to_pixel (self, self->priv->zoom_time) - self->priv->zoom_pos);
+		gtk_widget_queue_draw (self->priv->waveform);
+		render_cursor (self);
+		if (self->priv->has_selection) {
+			pt_waveviewer_selection_set (PT_WAVEVIEWER_SELECTION (self->priv->selection),
+					time_to_pixel (self, self->priv->sel_start),
+					time_to_pixel (self, self->priv->sel_end));
 		}
 		break;
 	default:
@@ -1341,16 +1157,32 @@ pt_waveviewer_set_property (GObject      *object,
 	}
 }
 
+static void
+adjustment_value_changed_cb (GtkAdjustment *adjustment,
+                             gpointer       user_data)
+{
+	PtWaveviewer *self = PT_WAVEVIEWER (user_data);
+
+	render_cursor (self);
+}
+
 static void
 pt_waveviewer_constructed (GObject *object)
 {
 	PtWaveviewer *self = PT_WAVEVIEWER (object);
+
+	GtkWidget *scrollbar = gtk_scrolled_window_get_hscrollbar (GTK_SCROLLED_WINDOW (self));
 	self->priv->adj = gtk_scrolled_window_get_hadjustment (GTK_SCROLLED_WINDOW (self));
-	g_signal_connect (gtk_scrolled_window_get_hscrollbar (GTK_SCROLLED_WINDOW (self)),
+
+	g_signal_connect (self->priv->adj,
+			  "value_changed",
+			  G_CALLBACK (adjustment_value_changed_cb),
+			  self);
+	g_signal_connect (scrollbar,
 			  "button_press_event",
 			  G_CALLBACK (scrollbar_button_press_event_cb),
 			  self);
-	g_signal_connect (gtk_scrolled_window_get_hscrollbar (GTK_SCROLLED_WINDOW (self)),
+	g_signal_connect (scrollbar,
 			  "scroll_event",
 			  G_CALLBACK (scrollbar_scroll_event_cb),
 			  self);
@@ -1376,7 +1208,15 @@ pt_waveviewer_init (PtWaveviewer *self)
 {
 	self->priv = pt_waveviewer_get_instance_private (self);
 
-	GtkWidget       *box, *overlay;
+	g_type_ensure (PT_TYPE_WAVEVIEWER_SCROLLBOX);
+	g_type_ensure (PT_TYPE_WAVEVIEWER_RULER);
+	g_type_ensure (PT_TYPE_WAVEVIEWER_WAVEFORM);
+	g_type_ensure (PT_TYPE_WAVEVIEWER_SELECTION);
+	g_type_ensure (PT_TYPE_WAVEVIEWER_CURSOR);
+	g_type_ensure (PT_TYPE_WAVEVIEWER_FOCUS);
+
+	gtk_widget_init_template (GTK_WIDGET (self));
+
 	GtkCssProvider  *provider;
 	GFile		*css_file;
 
@@ -1390,12 +1230,6 @@ pt_waveviewer_init (PtWaveviewer *self)
 	self->priv->zoom_time = 0;
 	self->priv->zoom_pos = 0;
 	self->priv->arrows = get_resize_cursor ();
-	self->priv->waveform = pt_waveviewer_waveform_new ();
-	self->priv->focus = pt_waveviewer_focus_new ();
-	self->priv->cursor = pt_waveviewer_cursor_new ();
-	self->priv->selection = pt_waveviewer_selection_new ();
-	self->priv->ruler = pt_waveviewer_ruler_new ();
-	self->priv->revealer = gtk_revealer_new ();
 	self->priv->loader = pt_waveloader_new (NULL);
 	self->priv->peaks = pt_waveloader_get_data (self->priv->loader);
 	self->priv->tick_handler = 0;
@@ -1404,29 +1238,6 @@ pt_waveviewer_init (PtWaveviewer *self)
 			PT_WAVEVIEWER_WAVEFORM (self->priv->waveform),
 			self->priv->peaks);
 
-	/* Setup scrolled window */
-	box = gtk_box_new (GTK_ORIENTATION_VERTICAL, 0);
-	overlay = gtk_overlay_new ();
-	gtk_revealer_set_transition_type (GTK_REVEALER (self->priv->revealer),
-					  GTK_REVEALER_TRANSITION_TYPE_SLIDE_UP);
-	gtk_revealer_set_transition_duration (GTK_REVEALER (self->priv->revealer), 200);
-	gtk_container_add (GTK_CONTAINER (overlay), self->priv->waveform);
-	gtk_overlay_add_overlay (GTK_OVERLAY (overlay), self->priv->selection);
-	gtk_overlay_add_overlay (GTK_OVERLAY (overlay), self->priv->cursor);
-	gtk_overlay_add_overlay (GTK_OVERLAY (overlay), self->priv->focus);
-	gtk_container_add (GTK_CONTAINER (box), overlay);
-	gtk_widget_set_vexpand (overlay, TRUE);
-	gtk_container_add (GTK_CONTAINER (self->priv->revealer), self->priv->ruler);
-	gtk_container_add (GTK_CONTAINER (box), self->priv->revealer);
-	gtk_container_add (GTK_CONTAINER (self), box);
-	gtk_scrolled_window_set_policy (
-			GTK_SCROLLED_WINDOW (self),
-			GTK_POLICY_ALWAYS,
-			GTK_POLICY_NEVER);
-	gtk_scrolled_window_set_shadow_type (
-			GTK_SCROLLED_WINDOW (self),
-			GTK_SHADOW_IN);
-	gtk_scrolled_window_set_overlay_scrolling (GTK_SCROLLED_WINDOW (self), FALSE);
 	gtk_widget_show_all (GTK_WIDGET (self));
 
 	css_file = g_file_new_for_uri ("resource:///org/parlatype/libparlatype/pt-waveviewer.css");
@@ -1440,14 +1251,8 @@ pt_waveviewer_init (PtWaveviewer *self)
 	g_object_unref (css_file);
 	g_object_unref (provider);
 
-	g_signal_connect (
-			GTK_WIDGET (self->priv->waveform),
-			"size-allocate",
-			G_CALLBACK (size_allocate_cb),
-			self);
-
 	/* Setup event handling */
-	self->priv->button = gtk_gesture_multi_press_new (box);
+	self->priv->button = gtk_gesture_multi_press_new (self->priv->scrollbox);
 	gtk_gesture_single_set_exclusive (GTK_GESTURE_SINGLE (self->priv->button), TRUE);
 	gtk_gesture_single_set_button (GTK_GESTURE_SINGLE (self->priv->button), 0);
 	gtk_event_controller_set_propagation_phase (GTK_EVENT_CONTROLLER (self->priv->button), GTK_PHASE_CAPTURE);
@@ -1463,7 +1268,7 @@ pt_waveviewer_init (PtWaveviewer *self)
 			self);
 
 #if GTK_CHECK_VERSION(3,24,0)
-	self->priv->motion_ctrl = gtk_event_controller_motion_new (box);
+	self->priv->motion_ctrl = gtk_event_controller_motion_new (self->priv->scrollbox);
 	g_signal_connect (
 			self->priv->motion_ctrl,
 			"motion",
@@ -1501,16 +1306,22 @@ pt_waveviewer_class_init (PtWaveviewerClass *klass)
 	gobject_class->dispose      = pt_waveviewer_dispose;
 	gobject_class->finalize     = pt_waveviewer_finalize;
 
-	widget_class->direction_changed    = pt_waveviewer_direction_changed;
 	widget_class->key_press_event      = pt_waveviewer_key_press_event;
 #if GTK_CHECK_VERSION(3,24,0)
 #else
 	widget_class->motion_notify_event  = pt_waveviewer_motion_notify_event;
 #endif
-	widget_class->realize              = pt_waveviewer_realize;
 	widget_class->scroll_event         = pt_waveviewer_scroll_event;
-	widget_class->state_flags_changed  = pt_waveviewer_state_flags_changed;
-	widget_class->style_updated        = pt_waveviewer_style_updated;
+
+	gtk_widget_class_set_template_from_resource (widget_class, "/org/parlatype/libparlatype/pt-waveviewer.ui");
+	gtk_widget_class_bind_template_child_private (widget_class, PtWaveviewer, scrollbox);
+	gtk_widget_class_bind_template_child_private (widget_class, PtWaveviewer, revealer);
+	gtk_widget_class_bind_template_child_private (widget_class, PtWaveviewer, ruler);
+	gtk_widget_class_bind_template_child_private (widget_class, PtWaveviewer, overlay);
+	gtk_widget_class_bind_template_child_private (widget_class, PtWaveviewer, waveform);
+	gtk_widget_class_bind_template_child_private (widget_class, PtWaveviewer, selection);
+	gtk_widget_class_bind_template_child_private (widget_class, PtWaveviewer, cursor);
+	gtk_widget_class_bind_template_child_private (widget_class, PtWaveviewer, focus);
 
 	/**
 	* PtWaveviewer::load-progress:
@@ -1639,17 +1450,18 @@ pt_waveviewer_class_init (PtWaveviewerClass *klass)
 		      0);
 
 	/**
-	* PtWaveviewer:playback-cursor:
-	*
-	* Current playback position in milliseconds.
-	*/
+	 * PtWaveviewer:playback-cursor:
+	 *
+	 * Current playback position in milliseconds. A value of -1 means an
+	 * invalid or unknown position that will not be rendered.
+	 */
 
 	obj_properties[PROP_PLAYBACK_CURSOR] =
 	g_param_spec_int64 (
 			"playback-cursor",
 			"Cursor position",
 			"Cursor’s position in 1/1000 seconds",
-			0,
+			-1,
 			G_MAXINT64,
 			0,
 			G_PARAM_WRITABLE | G_PARAM_STATIC_STRINGS);
