@@ -9,12 +9,12 @@
  * modify it under the terms of the GNU General Public License as published
  * by the Free Software Foundation, either version 3 of the License, or
  * (at your option) any later version.
- * 
+ *
  * This program is distributed in the hope that it will be useful, but
  * WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  * See the GNU General Public License for more details.
- * 
+ *
  * You should have received a copy of the GNU General Public License along
  * with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
@@ -23,7 +23,7 @@
 #include "config.h"
 #include <gtk/gtk.h>
 #include <glib/gi18n.h>
-#include <pt-player.h>
+#include <parlatype.h>
 #include "pt-goto-dialog.h"
 
 
@@ -87,7 +87,7 @@ output_cb (GtkSpinButton *spin,
 
 	if (dlg->priv->max == 0)
 		return GTK_INPUT_ERROR;
-	
+
 	text = pt_player_get_time_string (
 			gtk_spin_button_get_value_as_int (spin) * 1000,
 			dlg->priv->max, 0);
