@@ -4,12 +4,12 @@
  * modify it under the terms of the GNU General Public License as published
  * by the Free Software Foundation, either version 3 of the License, or
  * (at your option) any later version.
- * 
+ *
  * This program is distributed in the hope that it will be useful, but
  * WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  * See the GNU General Public License for more details.
- * 
+ *
  * You should have received a copy of the GNU General Public License along
  * with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
@@ -22,10 +22,8 @@
 #ifdef GDK_WINDOWING_X11
 #include <gdk/gdkx.h>
 #endif
-#include <pt-player.h>
-#include <pt-waveviewer.h>
+#include <parlatype.h>
 #include "pt-app.h"
-#include "pt-asr-assistant.h"
 #include "pt-goto-dialog.h"
 #include "pt-window.h"
 #include "pt-window-dnd.h"
@@ -35,8 +33,6 @@
 G_DEFINE_TYPE_WITH_PRIVATE (PtWindow, pt_window, GTK_TYPE_APPLICATION_WINDOW)
 
 static void play_button_toggled_cb (GtkToggleButton *button, PtWindow *win);
-static void jump_back_button_clicked_cb (GtkButton *button, PtWindow *win);
-static void jump_forward_button_clicked_cb (GtkButton *button, PtWindow *win);
 
 void
 pt_error_message (PtWindow    *parent,
@@ -57,8 +53,10 @@ pt_error_message (PtWindow    *parent,
 				GTK_MESSAGE_DIALOG (dialog),
 		                "%s", secondary_message);
 
-	gtk_dialog_run (GTK_DIALOG (dialog));
-	gtk_widget_destroy (dialog);
+	g_signal_connect_swapped (dialog, "response",
+			          G_CALLBACK (gtk_widget_destroy), dialog);
+
+	gtk_widget_show_all (dialog);
 }
 
 void
@@ -102,6 +100,24 @@ insert_timestamp (GSimpleAction *action,
 	gtk_clipboard_request_text (win->priv->clip, clip_text_cb, win);
 }
 
+static void
+goto_dialog_response_cb (GtkDialog *dlg,
+                         gint       response_id,
+                         gpointer   user_data)
+{
+	PtWindow *win = PT_WINDOW (user_data);
+	gint pos;
+
+	if (response_id == GTK_RESPONSE_OK) {
+		pos = pt_goto_dialog_get_pos (PT_GOTO_DIALOG (dlg));
+		pt_player_jump_to_position (win->player, pos * 1000);
+		pt_waveviewer_set_follow_cursor (PT_WAVEVIEWER (win->priv->waveviewer), TRUE);
+	}
+
+	gtk_widget_destroy (GTK_WIDGET (dlg));
+}
+
+
 void
 goto_position (GSimpleAction *action,
                GVariant      *parameter,
@@ -111,19 +127,15 @@ goto_position (GSimpleAction *action,
 	win = PT_WINDOW (user_data);
 
 	PtGotoDialog *dlg;
-	gint	      pos;
 
 	dlg = pt_goto_dialog_new (GTK_WINDOW (win));
 	pt_goto_dialog_set_pos (dlg, pt_player_get_position (win->player) / 1000);
 	pt_goto_dialog_set_max (dlg, pt_player_get_duration (win->player) / 1000);
 
-	if (gtk_dialog_run (GTK_DIALOG (dlg)) == GTK_RESPONSE_OK) {
-		pos = pt_goto_dialog_get_pos (dlg);
-		pt_player_jump_to_position (win->player, pos * 1000);
-		pt_waveviewer_set_follow_cursor (PT_WAVEVIEWER (win->priv->waveviewer), TRUE);
-	}
+	g_signal_connect (dlg, "response",
+			  G_CALLBACK (goto_dialog_response_cb), win);
 
-	gtk_widget_destroy (GTK_WIDGET (dlg));
+	gtk_widget_show_all (GTK_WIDGET (dlg));
 }
 
 void
@@ -137,6 +149,36 @@ goto_cursor (GSimpleAction *action,
 	pt_waveviewer_set_follow_cursor (PT_WAVEVIEWER (win->priv->waveviewer), TRUE);
 }
 
+void
+jump_back (GSimpleAction *action,
+           GVariant      *parameter,
+           gpointer       user_data)
+{
+	PtWindow *win = PT_WINDOW (user_data);
+
+	pt_player_jump_back (win->player);
+}
+
+void
+jump_forward (GSimpleAction *action,
+              GVariant      *parameter,
+              gpointer       user_data)
+{
+	PtWindow *win = PT_WINDOW (user_data);
+
+	pt_player_jump_forward (win->player);
+}
+
+void
+play (GSimpleAction *action,
+      GVariant      *parameter,
+      gpointer       user_data)
+{
+	PtWindow *win = PT_WINDOW (user_data);
+
+	pt_player_play_pause (win->player);
+}
+
 static void
 set_zoom (GSettings *editor,
           gint       step)
@@ -168,124 +210,29 @@ zoom_out (GSimpleAction *action,
 	set_zoom (win->priv->editor, -25);
 }
 
-#ifdef HAVE_ASR
-static void
-setup_sphinx (PtWindow *win)
+static gboolean
+setup_asr (PtWindow *win)
 {
 	GError *error = NULL;
+	gboolean success;
 
-	pt_asr_settings_apply_config (
-			win->priv->asr_settings,
-			g_settings_get_string (win->priv->editor, "asr-config"),
-			win->player);
+	success = pt_player_configure_asr (
+			win->player,
+			win->priv->asr_config,
+			&error);
 
-	pt_player_setup_sphinx (win->player, &error);
-	if (error) {
+	if (success)
+		pt_player_set_mode (win->player, PT_MODE_ASR);
+	else
 		pt_error_message (win, error->message, NULL);
-		g_clear_error (&error);
-	}
-}
 
-static void
-asr_output_search_cancelled_cb (PtAsrOutput *output,
-                                PtWindow    *win)
-{
-	GAction  *action;
-	GVariant *state;
-
-	g_signal_handler_disconnect (win->priv->output, win->priv->output_handler_id1);
-	g_signal_handler_disconnect (win->priv->output, win->priv->output_handler_id2);
-	g_clear_object (&win->priv->output);
-
-	action = g_action_map_lookup_action (G_ACTION_MAP (win), "mode");
-	state = g_variant_new_string ("playback");
-	g_simple_action_set_state (G_SIMPLE_ACTION (action), state);
-}
-
-static void
-asr_output_app_found_cb (PtAsrOutput *output,
-                         PtWindow    *win)
-{
-	GApplication  *app;
-	GNotification *notification;
-	gchar         *summary;
-	gchar         *message;
-
-	g_signal_handler_disconnect (win->priv->output, win->priv->output_handler_id1);
-	g_signal_handler_disconnect (win->priv->output, win->priv->output_handler_id2);
-
-	summary = g_strdup_printf (_("Found %s"), pt_asr_output_get_app_name (output));
-	message = _("Press “Play” to start automatic speech recognition.");
-	notification = g_notification_new (summary);
-	g_notification_set_body (notification, message);
-	app = G_APPLICATION (gtk_window_get_application (GTK_WINDOW (win)));
-	g_application_send_notification (app, "app-found", notification);
-
-	setup_sphinx (win);
-
-	g_application_withdraw_notification (app, "app-found");
-	g_object_unref (notification);
-	g_free (summary);
-}
-
-static void
-set_mode_asr (PtWindow *win)
-{
-	win->priv->output = pt_asr_output_new ();
-	win->priv->output_handler_id1 = g_signal_connect (
-			win->priv->output, "app-found",
-			G_CALLBACK (asr_output_app_found_cb), win);
-	win->priv->output_handler_id2 = g_signal_connect (
-			win->priv->output, "search-cancelled",
-			G_CALLBACK (asr_output_search_cancelled_cb), win);
-	pt_asr_output_search_app (win->priv->output, GTK_WINDOW (win));
-}
-
-static void
-asr_assistant_cancel_cb (GtkAssistant *assistant,
-                         gpointer      user_data)
-{
-	gtk_widget_destroy (GTK_WIDGET (assistant));
-}
-
-static void
-asr_assistant_close_cb (GtkAssistant *assistant,
-                        PtWindow     *win)
-{
-	gchar            *name;
-	gchar            *id;
-
-	name = pt_asr_assistant_get_name (PT_ASR_ASSISTANT (assistant));
-	id = pt_asr_assistant_save_config (
-			PT_ASR_ASSISTANT (assistant), win->priv->asr_settings);
-	g_settings_set_string (win->priv->editor, "asr-config", id);
-
-	gtk_widget_destroy (GTK_WIDGET (assistant));
-	g_free (name);
-	g_free (id);
-}
-
-static void
-launch_asr_assistant (PtWindow *win)
-{
-	GtkWidget *assistant;
-
-	assistant = GTK_WIDGET (pt_asr_assistant_new (GTK_WINDOW (win)));
-	g_signal_connect (assistant, "cancel", G_CALLBACK (asr_assistant_cancel_cb), NULL);
-	g_signal_connect (assistant, "close", G_CALLBACK (asr_assistant_close_cb), win);
-	gtk_widget_show (assistant);
+	return success;
 }
 
 static void
 set_mode_playback (PtWindow *win)
 {
-	GError *error = NULL;
-
-	pt_player_setup_player (win->player, &error);
-	if (error) {
-		pt_error_message (win, error->message, NULL);
-		g_clear_error (&error);
-	}
+	pt_player_set_mode (win->player, PT_MODE_PLAYBACK);
 }
 
 void
@@ -295,30 +242,26 @@ change_mode (GSimpleAction *action,
 {
 	PtWindow *win = PT_WINDOW (user_data);
 	const gchar *mode;
+	gboolean success = TRUE;
 
-	if (!win->priv->asr_settings) {
-		GtkApplication *app;
-		app = gtk_window_get_application (GTK_WINDOW (win));
-		win->priv->asr_settings = g_object_ref (pt_app_get_asr_settings (PT_APP (app)));
-	}
+	/* Work around synchronisation issues in PtPlayer in playing state */
+	pt_player_pause (win->player);
 
 	mode = g_variant_get_string (state, NULL);
 	if (g_strcmp0 (mode, "playback") == 0) {
-		g_clear_object (&win->priv->output);
 		set_mode_playback (win);
 	} else if (g_strcmp0 (mode, "asr") == 0) {
-		if (pt_asr_settings_have_configs (win->priv->asr_settings)) {
-			/* TODO && have config saved in GSettings */
-			set_mode_asr (win);
+		if (win->priv->asr_config != NULL) {
+			success = setup_asr (win);
 		} else {
-			launch_asr_assistant (win);
 			return;
 		}
 	} else {
 		g_assert_not_reached ();
 	}
 
-	g_simple_action_set_state (action, state);
+	if (success)
+		g_simple_action_set_state (action, state);
 
 	/* On changing state the menu doesn’t close. It seems better to close
 	   it just like every other menu item closes the menu.
@@ -328,18 +271,18 @@ change_mode (GSimpleAction *action,
 	if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (win->priv->primary_menu_button)))
 		gtk_button_clicked (GTK_BUTTON (win->priv->primary_menu_button));
 }
-#endif
 
 const GActionEntry win_actions[] = {
-#ifdef HAVE_ASR
 	{ "mode", NULL, "s", "'playback'", change_mode },
-#endif
 	{ "copy", copy_timestamp, NULL, NULL, NULL },
 	{ "insert", insert_timestamp, NULL, NULL, NULL },
 	{ "goto", goto_position, NULL, NULL, NULL },
 	{ "goto-cursor", goto_cursor, NULL, NULL, NULL },
 	{ "zoom-in", zoom_in, NULL, NULL, NULL },
-	{ "zoom-out", zoom_out, NULL, NULL, NULL }
+	{ "zoom-out", zoom_out, NULL, NULL, NULL },
+	{ "jump-back", jump_back, NULL, NULL, NULL },
+	{ "jump-forward", jump_forward, NULL, NULL, NULL },
+	{ "play", play, NULL, NULL, NULL }
 };
 
 static void
@@ -350,6 +293,9 @@ update_time (PtWindow *win)
 
 	time = pt_player_get_position (win->player);
 
+	if (time == -1)
+		return;
+
 	/* Update label only if time has changed at 10th seconds level */
 	if (time/100 != win->priv->last_time) {
 		text = pt_player_get_current_time_string (
@@ -564,11 +510,9 @@ enable_win_actions (PtWindow *win,
 {
 	GAction *action;
 
-#ifdef HAVE_ASR
 	/* always active */
 	action = g_action_map_lookup_action (G_ACTION_MAP (win), "mode");
 	g_simple_action_set_enabled (G_SIMPLE_ACTION (action), TRUE);
-#endif
 
 	action = g_action_map_lookup_action (G_ACTION_MAP (win), "copy");
 	g_simple_action_set_enabled (G_SIMPLE_ACTION (action), state);
@@ -584,7 +528,7 @@ enable_win_actions (PtWindow *win,
 	action = g_action_map_lookup_action (G_ACTION_MAP (win), "goto");
 	g_simple_action_set_enabled (G_SIMPLE_ACTION (action), state);
 
-	/* always insensitve: either there is no waveform or we are already at cursor position */
+	/* always insensitive: either there is no waveform or we are already at cursor position */
 	action = g_action_map_lookup_action (G_ACTION_MAP (win), "goto-cursor");
 	g_simple_action_set_enabled (G_SIMPLE_ACTION (action), FALSE);
 
@@ -610,8 +554,6 @@ pt_window_ready_to_play (PtWindow *win,
 	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (win->priv->button_play), FALSE);
 
 	gtk_widget_set_sensitive (win->priv->button_play, state);
-	gtk_widget_set_sensitive (win->priv->button_fast_back, state);
-	gtk_widget_set_sensitive (win->priv->button_fast_forward, state);
 	gtk_widget_set_sensitive (win->priv->button_jump_back, state);
 	gtk_widget_set_sensitive (win->priv->button_jump_forward, state);
 	gtk_widget_set_sensitive (win->priv->speed_scale, state);
@@ -655,7 +597,6 @@ player_error_cb (PtPlayer *player,
 {
 	pt_window_ready_to_play (win, FALSE);
 	pt_error_message (win, error->message, NULL);
-	/* TODO clear error? */
 }
 
 static void
@@ -669,8 +610,8 @@ open_cb (PtWaveviewer *viewer,
 	if (!pt_waveviewer_load_wave_finish (viewer, res, &error)) {
 		pt_error_message (win, error->message, NULL);
 		g_error_free (error);
-		/* TODO that's it? */
-		return;
+		/* Very unlikely situation: Stream is open and playable,
+		 * but loading the waveform failed. */
 	}
 }
 
@@ -753,190 +694,98 @@ static void
 player_end_of_stream_cb (PtPlayer *player,
                          PtWindow *win)
 {
-	/* Don’t jump back */
-	GtkToggleButton *play;
-	play = GTK_TOGGLE_BUTTON (win->priv->button_play);
-
-	g_signal_handlers_block_by_func (play, play_button_toggled_cb, win);
-	gtk_toggle_button_set_active (play, FALSE);
-	g_signal_handlers_unblock_by_func (play, play_button_toggled_cb, win);
+	/* Pause player without jumping back, this will also toggle the
+	 * Play button */
 	pt_player_pause (win->player);
 	change_play_button_tooltip (win);
 }
 
-static void
-player_jumped_back_cb (PtPlayer *player,
-                       PtWindow *win)
-{
-	GtkButton *button;
-	button = GTK_BUTTON (win->priv->button_jump_back);
-	g_signal_handlers_block_by_func (button, jump_back_button_clicked_cb, win);
-	gtk_button_clicked (button);
-	g_signal_handlers_unblock_by_func (button, jump_back_button_clicked_cb, win);
-}
-
-static void
-jump_back_button_clicked_cb (GtkButton *button,
-                             PtWindow  *win)
-{
-	g_signal_handlers_block_by_func (win->player, player_jumped_back_cb, win);
-	pt_player_jump_back (win->player);
-	g_signal_handlers_unblock_by_func (win->player, player_jumped_back_cb, win);
-}
-
-static void
-player_jumped_forward_cb (PtPlayer *player,
-                          PtWindow *win)
-{
-	GtkButton *button;
-	button = GTK_BUTTON (win->priv->button_jump_forward);
-	g_signal_handlers_block_by_func (button, jump_forward_button_clicked_cb, win);
-	gtk_button_clicked (button);
-	g_signal_handlers_unblock_by_func (button, jump_forward_button_clicked_cb, win);
-}
-
-static void
-jump_forward_button_clicked_cb (GtkButton *button,
-                                PtWindow  *win)
-{
-	pt_player_jump_forward (win->player);
-}
-
-/* currently not used */
 static gboolean
-fast_back_button_pressed_cb (GtkButton *button,
-                             GdkEvent  *event,
-                             PtWindow  *win)
+swap_control_buttons (GtkWidget *box)
 {
-	pt_player_rewind (win->player, 2.0);
+	gtk_widget_set_direction (box, GTK_TEXT_DIR_LTR);
 	return FALSE;
 }
 
-/* currently not used */
-static gboolean
-fast_back_button_released_cb (GtkButton *button,
-                              GdkEvent  *event,
-                              PtWindow  *win)
-{
-	pt_player_set_speed (win->player, win->priv->speed);
-	if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (win->priv->button_play))) {
-		pt_player_play (win->player);
-	} else {
-		pt_player_pause (win->player);
-	}
-	return FALSE;
-}
-
-/* currently not used */
-static gboolean
-fast_forward_button_pressed_cb (GtkButton *button,
-                                GdkEvent  *event,
-                                PtWindow  *win)
-{
-	pt_player_fast_forward (win->player, 2.0);
-	return FALSE;
-}
-
-/* currently not used */
-static gboolean
-fast_forward_button_released_cb (GtkButton *button,
-                                 GdkEvent  *event,
-                                 PtWindow  *win)
-{
-	pt_player_set_speed (win->player, win->priv->speed);
-	if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (win->priv->button_play))) {
-		pt_player_play (win->player);
-	} else {
-		pt_player_pause (win->player);
-	}
-	return FALSE;
-}
-
-#ifdef HAVE_ASR
 static void
-player_asr_final_cb (PtPlayer *player,
-                     gchar    *word,
-                     PtWindow *win)
+pt_window_direction_changed (GtkWidget        *widget,
+                             GtkTextDirection  previous_direction)
 {
-	pt_asr_output_final (win->priv->output, word);
-}
+	/* In RTL layouts playback control elements are *not* supposed to be
+	 * mirrored. Undo automatic mirroring for those elements. */
 
-static void
-player_asr_hypothesis_cb (PtPlayer *player,
-                          gchar    *word,
-                          PtWindow *win)
-{
-	pt_asr_output_hypothesis (win->priv->output, word);
-}
-#endif
+	PtWindow        *self = PT_WINDOW (widget);
+	PtWindowPrivate *priv = self->priv;
+	GtkScale        *speed_scale = GTK_SCALE (priv->speed_scale);
 
-static void
-speed_scale_direction_changed_cb (GtkWidget        *widget,
-                                  GtkTextDirection  previous_direction,
-                                  gpointer          data)
-{
-	if (gtk_widget_get_direction (widget) == GTK_TEXT_DIR_RTL)
-		gtk_scale_set_value_pos (GTK_SCALE (widget), GTK_POS_LEFT);
+	gtk_widget_set_direction (priv->controls_row_box, GTK_TEXT_DIR_LTR);
+	gtk_widget_set_direction (priv->progress, GTK_TEXT_DIR_LTR);
+
+	if (previous_direction == GTK_TEXT_DIR_LTR)
+		gtk_scale_set_value_pos (speed_scale, GTK_POS_LEFT);
 	else
-		gtk_scale_set_value_pos (GTK_SCALE (widget), GTK_POS_RIGHT);
+		gtk_scale_set_value_pos (speed_scale, GTK_POS_RIGHT);
+
+	/* Swap control buttons in a timeout to avoid a race condition where
+	 * buttons were not rendered correctly linked, reason unknown */
+	g_idle_add ((GSourceFunc) swap_control_buttons, priv->controls_box);
 }
 
 static void
-swap_accelerators (GtkWidget     *widget,
-                   GtkAccelGroup *accels,
-                   guint          old,
-                   guint          new)
+set_asr_config (PtWindow *win)
 {
-	gtk_widget_remove_accelerator (
-			widget,
-			accels,
-			old,
-			GDK_CONTROL_MASK);
-	gtk_widget_add_accelerator (
-			widget,
-			"clicked",
-			accels,
-			new,
-			GDK_CONTROL_MASK,
-			GTK_ACCEL_VISIBLE);
-}
+	PtWindowPrivate *priv = win->priv;
+	GFile    *asr_file;
+	gchar    *asr_path;
+	GAction  *action;
+	GVariant *variant;
+	const gchar *mode;
 
-static void
-jump_back_direction_changed_cb (GtkWidget        *widget,
-                                GtkTextDirection  previous_direction,
-                                gpointer          data)
-{
-	PtWindow *self = PT_WINDOW (data);
-	guint old, new;
+	/* called when config file changed (and on startup);
+	 * get rid of old config object first */
+	if (priv->asr_config)
+		g_clear_object (&priv->asr_config);
 
-	if (previous_direction == GTK_TEXT_DIR_LTR) {
-		old = GDK_KEY_Left;
-		new = GDK_KEY_Right;
-	} else {
-		old = GDK_KEY_Right;
-		new = GDK_KEY_Left;
+	/* get new config object */
+	asr_path = g_settings_get_string (priv->editor, "asr-config");
+	asr_file = g_file_new_for_path (asr_path);
+	priv->asr_config = pt_config_new (asr_file);
+	g_object_unref (asr_file);
+	g_free (asr_path);
+
+	/* get current mode */
+	action = g_action_map_lookup_action (G_ACTION_MAP (win), "mode");
+	variant = g_action_get_state (action);
+	mode = g_variant_get_string (variant, NULL);
+
+	/* not valid: clear object, remove menu, change to playback mode */
+	if (!pt_config_is_valid (priv->asr_config)                        ||
+	    !pt_player_config_is_loadable (win->player, priv->asr_config) ||
+	    !pt_config_is_installed (priv->asr_config)) {
+		g_clear_object (&priv->asr_config);
+		if (priv->asr) {
+			g_menu_remove (G_MENU (priv->primary_menu), 0);
+			priv->asr = FALSE;
+		}
+		if (g_strcmp0 (mode, "asr") == 0) {
+			set_mode_playback (win);
+		}
+		g_variant_unref (variant);
+		return;
 	}
 
-	swap_accelerators (widget, self->priv->accels, old, new);
-}
-
-static void
-jump_forward_direction_changed_cb (GtkWidget        *widget,
-                                   GtkTextDirection  previous_direction,
-                                   gpointer          data)
-{
-	PtWindow *self = PT_WINDOW (data);
-	guint old, new;
-
-	if (previous_direction == GTK_TEXT_DIR_LTR) {
-		old = GDK_KEY_Right;
-		new = GDK_KEY_Left;
-	} else {
-		old = GDK_KEY_Left;
-		new = GDK_KEY_Right;
+	/* valid: add menu, setup ASR plugin */
+	if (!priv->asr) {
+		g_menu_prepend_section (G_MENU (priv->primary_menu),
+		                        NULL, /* label */
+		                        G_MENU_MODEL (priv->asr_menu));
+		priv->asr = TRUE;
 	}
 
-	swap_accelerators (widget, self->priv->accels, old, new);
+	if (g_strcmp0 (mode, "asr") == 0)
+		setup_asr (win);
+
+	g_variant_unref (variant);
 }
 
 static void
@@ -958,6 +807,11 @@ settings_changed_cb (GSettings *settings,
 		change_jump_forward_tooltip (win);
 		return;
 	}
+
+	if (g_strcmp0 (key, "asr-config") == 0) {
+		set_asr_config (win);
+		return;
+	}
 }
 
 static gboolean
@@ -1071,6 +925,9 @@ setup_settings (PtWindow *win)
 			win->player, "timestamp-fraction-sep",
 			G_SETTINGS_BIND_GET);
 
+	win->priv->asr = FALSE;
+	set_asr_config (win);
+
 	/* connect to tooltip changer */
 
 	g_signal_connect (
@@ -1187,19 +1044,8 @@ volume_button_press_event (GtkGestureMultiPress *gesture,
 static void
 setup_player (PtWindow *win)
 {
-	GError *error = NULL;
-
 	win->player = pt_player_new ();
-	if (!pt_player_setup_player (win->player, &error)) {
-		gchar *secondary_message = g_strdup_printf (
-			_("Parlatype needs GStreamer 1.x to run. Please check your installation of "
-                        "GStreamer and make sure you have the “Good Plugins” installed.\n"
-                        "Parlatype will quit now, it received this error message: %s"), error->message);
-		pt_error_message (win, _("Fatal error"), secondary_message);
-		g_clear_error (&error);
-		g_free (secondary_message);
-		exit (2);
-	}
+	pt_player_set_mode (win->player, PT_MODE_PLAYBACK);
 
 	pt_player_connect_waveviewer (
 			win->player,
@@ -1220,34 +1066,12 @@ setup_player (PtWindow *win)
 			G_CALLBACK (player_play_toggled_cb),
 			win);
 
-	g_signal_connect (win->player,
-			"jumped-back",
-			G_CALLBACK (player_jumped_back_cb),
-			win);
-
-	g_signal_connect (win->player,
-			"jumped-forward",
-			G_CALLBACK (player_jumped_forward_cb),
-			win);
-
 	GtkAdjustment *speed_adjustment;
 	speed_adjustment = gtk_range_get_adjustment (GTK_RANGE (win->priv->speed_scale));
 	g_object_bind_property (
 			speed_adjustment, "value",
 			win->player, "speed",
 			G_BINDING_BIDIRECTIONAL | G_BINDING_SYNC_CREATE);
-
-#ifdef HAVE_ASR
-	g_signal_connect (win->player,
-			"asr-final",
-			G_CALLBACK (player_asr_final_cb),
-			win);
-
-	g_signal_connect (win->player,
-			"asr-hypothesis",
-			G_CALLBACK (player_asr_hypothesis_cb),
-			win);
-#endif
 }
 
 static void
@@ -1302,9 +1126,9 @@ setup_volume (PtWindow *win)
 }
 
 static void
-setup_accels_actions_headerbar (PtWindow *win)
+setup_accels_actions_menus (PtWindow *win)
 {
-	/* Actions */	
+	/* Actions */
 	g_action_map_add_action_entries (G_ACTION_MAP (win),
 				win_actions,
 				G_N_ELEMENTS (win_actions),
@@ -1312,78 +1136,26 @@ setup_accels_actions_headerbar (PtWindow *win)
 
 	enable_win_actions (win, FALSE);
 
-	/* GtkHeader workaround for glade 3.16 + Menu button */
-	GtkBuilder    *builder;
-	GtkWidget     *hbar;
-	GMenuModel    *primary_menu;
-
-	builder = gtk_builder_new_from_resource ("/org/parlatype/parlatype/window-headerbar.ui");
-	hbar = GTK_WIDGET (gtk_builder_get_object (builder, "headerbar"));
-	win->priv->button_open = GTK_WIDGET (gtk_builder_get_object (builder, "button_open"));
-	win->priv->primary_menu_button = GTK_WIDGET (gtk_builder_get_object (builder, "primary_menu_button"));
-	gtk_window_set_titlebar (GTK_WINDOW (win), hbar);
-	gtk_builder_connect_signals (builder, win);
-	g_object_unref (builder);
+	/* Setup menus */
+	PtWindowPrivate *priv = win->priv;
+	GtkBuilder      *builder;
 
 	builder = gtk_builder_new_from_resource ("/org/parlatype/parlatype/menus.ui");
-	primary_menu = G_MENU_MODEL (gtk_builder_get_object (builder, "primary-menu"));
-	win->priv->secondary_menu = G_MENU_MODEL (gtk_builder_get_object (builder, "secondary-menu"));
-	gtk_menu_button_set_menu_model (GTK_MENU_BUTTON (win->priv->primary_menu_button), primary_menu);
-	gtk_menu_button_set_menu_model (GTK_MENU_BUTTON (win->priv->pos_menu_button), win->priv->secondary_menu);
+	priv->primary_menu   = G_MENU_MODEL (gtk_builder_get_object (builder, "primary-menu"));
+	priv->secondary_menu = G_MENU_MODEL (gtk_builder_get_object (builder, "secondary-menu"));
+	gtk_menu_button_set_menu_model (GTK_MENU_BUTTON (priv->primary_menu_button), priv->primary_menu);
+	gtk_menu_button_set_menu_model (GTK_MENU_BUTTON (priv->pos_menu_button), priv->secondary_menu);
 	g_object_unref (builder);
-	win->priv->go_to_timestamp = g_menu_item_new (_("Go to time in clipboard"), "win.insert");
-	g_menu_insert_item (G_MENU (win->priv->secondary_menu), 1, win->priv->go_to_timestamp);
 
-	GApplication *app;
-	app = g_application_get_default ();
-	if (!pt_app_get_asr (PT_APP (app))) {
-		g_menu_remove (G_MENU (primary_menu), 0);
-	}
+	/* Setup ASR menu manually; no success with GtkBuilder */
+	priv->asr_menu = g_menu_new ();
+	priv->asr_menu_item1 = g_menu_item_new (_("Manual transcription"), "win.mode::playback");
+	priv->asr_menu_item2 = g_menu_item_new (_("Automatic transcription"), "win.mode::asr");
+	g_menu_append_item (priv->asr_menu, priv->asr_menu_item1);
+	g_menu_append_item (priv->asr_menu, priv->asr_menu_item2);
 
-	/* Accels */
-	win->priv->accels = gtk_accel_group_new ();
-	gtk_window_add_accel_group (GTK_WINDOW (win), win->priv->accels);
-	
-	gtk_widget_add_accelerator (
-			win->priv->primary_menu_button,
-			"clicked",
-			win->priv->accels,
-			GDK_KEY_F10,
-			0,
-			GTK_ACCEL_VISIBLE);
-
-	guint jump_back, jump_forward;
-	if (gtk_widget_get_default_direction () == GTK_TEXT_DIR_LTR) {
-		jump_back = GDK_KEY_Left;
-		jump_forward = GDK_KEY_Right;
-	} else {
-		jump_back = GDK_KEY_Right;
-		jump_forward = GDK_KEY_Left;
-	}
-
-	gtk_widget_add_accelerator (
-			win->priv->button_jump_back,
-			"clicked",
-			win->priv->accels,
-			jump_back,
-			GDK_CONTROL_MASK,
-			GTK_ACCEL_VISIBLE);
-
-	gtk_widget_add_accelerator (
-			win->priv->button_jump_forward,
-			"clicked",
-			win->priv->accels,
-			jump_forward,
-			GDK_CONTROL_MASK,
-			GTK_ACCEL_VISIBLE);
-
-	gtk_widget_add_accelerator (
-			win->priv->button_play,
-			"clicked",
-			win->priv->accels,
-			GDK_KEY_space,
-			GDK_CONTROL_MASK,
-			GTK_ACCEL_VISIBLE);
+	priv->go_to_timestamp = g_menu_item_new (_("Go to time in clipboard"), "win.insert");
+	g_menu_insert_item (G_MENU (win->priv->secondary_menu), 1, priv->go_to_timestamp);
 }
 
 static void
@@ -1394,16 +1166,11 @@ pt_window_init (PtWindow *win)
 	gtk_widget_init_template (GTK_WIDGET (win));
 
 	setup_player (win);
+	setup_accels_actions_menus (win);
 	setup_settings (win);
-	setup_accels_actions_headerbar (win);
 	setup_volume (win);
 	pt_window_setup_dnd (win);	/* this is in pt_window_dnd.c */
-#ifdef HAVE_ASR
-	win->priv->asr_settings = NULL;
-	win->priv->output = NULL;
-	win->priv->output_handler_id1 = 0;
-	win->priv->output_handler_id2 = 0;
-#endif
+
 	win->priv->recent = gtk_recent_manager_get_default ();
 	win->priv->timer = 0;
 	win->priv->last_time = 0;
@@ -1413,9 +1180,9 @@ pt_window_init (PtWindow *win)
 	/* Used e.g. by Xfce */
 	gtk_window_set_default_icon_name (APP_ID);
 
-	/* Flip speed scale for right to left layouts */
+	/* Prepare layout if we have initially RTL */
 	if (gtk_widget_get_default_direction () == GTK_TEXT_DIR_RTL)
-		gtk_scale_set_value_pos (GTK_SCALE (win->priv->speed_scale), GTK_POS_LEFT);
+		pt_window_direction_changed (GTK_WIDGET (win), GTK_TEXT_DIR_LTR);
 
 	pt_window_ready_to_play (win, FALSE);
 
@@ -1459,12 +1226,16 @@ pt_window_dispose (GObject *object)
 	remove_timer (win);
 	g_clear_object (&win->priv->editor);
 	g_clear_object (&win->player);
-#ifdef HAVE_ASR
-	g_clear_object (&win->priv->output);
-	g_clear_object (&win->priv->asr_settings);
-#endif
+	g_clear_object (&win->priv->asr_config);
 	g_clear_object (&win->priv->go_to_timestamp);
 	g_clear_object (&win->priv->vol_event);
+	g_clear_object (&win->priv->asr_menu);
+	g_clear_object (&win->priv->asr_menu_item1);
+	g_clear_object (&win->priv->asr_menu_item2);
+	if (win->priv->vol_icons) {
+		g_strfreev (win->priv->vol_icons);
+		win->priv->vol_icons = NULL;
+	}
 
 	G_OBJECT_CLASS (pt_window_parent_class)->dispose (object);
 }
@@ -1475,24 +1246,19 @@ pt_window_class_init (PtWindowClass *klass)
 	GObjectClass   *gobject_class = G_OBJECT_CLASS (klass);
 	GtkWidgetClass *widget_class  = GTK_WIDGET_CLASS (klass);
 
-	gobject_class->dispose      = pt_window_dispose;
+	gobject_class->dispose          = pt_window_dispose;
+	widget_class->direction_changed = pt_window_direction_changed;
+
 	gtk_widget_class_set_template_from_resource (widget_class, "/org/parlatype/parlatype/window.ui");
-	gtk_widget_class_bind_template_callback(widget_class, fast_back_button_pressed_cb);
-	gtk_widget_class_bind_template_callback(widget_class, fast_back_button_released_cb);
-	gtk_widget_class_bind_template_callback(widget_class, fast_forward_button_pressed_cb);
-	gtk_widget_class_bind_template_callback(widget_class, fast_forward_button_released_cb);
-	gtk_widget_class_bind_template_callback(widget_class, jump_back_button_clicked_cb);
-	gtk_widget_class_bind_template_callback(widget_class, jump_back_direction_changed_cb);
-	gtk_widget_class_bind_template_callback(widget_class, jump_forward_button_clicked_cb);
-	gtk_widget_class_bind_template_callback(widget_class, jump_forward_direction_changed_cb);
 	gtk_widget_class_bind_template_callback(widget_class, play_button_toggled_cb);
-	gtk_widget_class_bind_template_callback(widget_class, speed_scale_direction_changed_cb);
 	gtk_widget_class_bind_template_callback(widget_class, zoom_in_cb);
 	gtk_widget_class_bind_template_callback(widget_class, zoom_out_cb);
+	gtk_widget_class_bind_template_child_private (widget_class, PtWindow, primary_menu_button);
+	gtk_widget_class_bind_template_child_private (widget_class, PtWindow, button_open);
 	gtk_widget_class_bind_template_child_private (widget_class, PtWindow, progress);
+	gtk_widget_class_bind_template_child_private (widget_class, PtWindow, controls_row_box);
+	gtk_widget_class_bind_template_child_private (widget_class, PtWindow, controls_box);
 	gtk_widget_class_bind_template_child_private (widget_class, PtWindow, button_play);
-	gtk_widget_class_bind_template_child_private (widget_class, PtWindow, button_fast_back);	// not used
-	gtk_widget_class_bind_template_child_private (widget_class, PtWindow, button_fast_forward);	// not used
 	gtk_widget_class_bind_template_child_private (widget_class, PtWindow, button_jump_back);
 	gtk_widget_class_bind_template_child_private (widget_class, PtWindow, button_jump_forward);
 	gtk_widget_class_bind_template_child_private (widget_class, PtWindow, volumebutton);
